import mxupy as mu
from bigOAINet.db.data.m.City import City

class CityControl(mu.EntityXControl):
    class Meta:
        model_class = City
    
    def get_id(self, name):
        """
        获取编号

        参数:
            name (str): 城市名称

        返回:
            int: 城市编号，如果未找到则返回-1
        """
        def _do(im):
            mc = self.model_class
            im.data = -1
            im.data = mc.get(mc.name.contains(name)).cityId

        return self.run(_do)

    def add_city(self, id, provinceId, name, enname):
        """
        添加（用于初始化数据）

        参数:
            id (int): 区域id
            provinceId (int): 省份id
            name (str): 名称
            enName (str): 英文名称

        返回:
            IM: 返回操作结果
        """
        def _do(im):
            
            mc = self.model_class
            
            if mc.select().where(mc.cityId == id).exists():
                im.success = False
                im.msg = f'已经存在 cityId 为 {id} 的记录。'
                return
            
            c = {
                'cityId':id,
                'provinceId':provinceId,
                'name':name,
                'enname':enname
            }    
            im.data = mc.create(**c)

        return self.run(_do)
    
    
    def init_data(self):
        """
        初始化数据

        参数:

        返回:
            IM: 返回操作结果
        """
        
        cs = [
            (1034, 1015, "马鞍山市", "Maanshan"),
            (1035, 1015, "六安市", "Liuan"),
            (1036, 1015, "宣城市", "Xuancheng"),
            (1037, 1015, "宿州市", "Suzhou"),
            (1038, 1015, "铜陵市", "Tongling"),
            (1039, 1015, "芜湖市", "Wuhu"),
            (1040, 1015, "亳州市", "Bozhou"),
            (1041, 1015, "蚌埠市", "Bengbu"),
            (1042, 1015, "安庆市", "Anqing"),
            (1043, 1015, "滁州市", "Chuzhou"),
            (1044, 1015, "巢湖市", "Caohu"),
            (1045, 1015, "阜阳市", "Fuyang"),
            (1046, 1015, "池州市", "Chizhou"),
            (1047, 1015, "合肥市", "Hefei"),
            (1048, 1015, "黄山市", "Huangshan"),
            (1049, 1015, "淮南市", "Huainan"),
            (1050, 1015, "淮北市", "Huaibei"),
            (1051, 1016, "龙岩市", "Longyan"),
            (1052, 1016, "三明市", "Sanming"),
            (1053, 1016, "泉州市", "Quanzhou"),
            (1054, 1016, "厦门市", "Xiamen"),
            (1055, 1016, "漳州市", "Zhangzhou"),
            (1056, 1016, "莆田市", "Putian"),
            (1057, 1016, "南平市", "Manping"),
            (1058, 1016, "宁德市", "Ningde"),
            (1059, 1016, "福州市", "Fuzhou"),
            (1060, 1024, "临夏回族自治州", "Linxia"),
            (1061, 1024, "陇南地区", "Longnan"),
            (1062, 1024, "兰州市", "Lanzhou"),
            (1063, 1024, "天水市", "Tianshui"),
            (1064, 1024, "武威市", "Wuwei"),
            (1065, 1024, "庆阳市", "Qingyang"),
            (1066, 1024, "张掖市", "Zhangye"),
            (1067, 1024, "平凉市", "Pingliang"),
            (1068, 1024, "白银市", "Baiyin"),
            (1069, 1024, "定西市", "Dingxi"),
            (1070, 1024, "甘南藏族自治州", "Gannan"),
            (1071, 1024, "嘉峪关市", "Jiayuguan"),
            (1072, 1024, "金昌市", "Jinchang"),
            (1073, 1024, "酒泉市", "Jiuquan"),
            (1074, 1005, "茂名市", "Maoming"),
            (1075, 1005, "梅州市", "Meizhou"),
            (1076, 1005, "清远市", "Qingyuan"),
            (1077, 1005, "汕尾市", "Shanwei"),
            (1078, 1005, "汕头市", "Shantou"),
            (1079, 1005, "深圳市", "Shenzhen"),
            (1081, 1005, "韶关市", "Shaoguan"),
            (1082, 1005, "阳江市", "Yangjiang"),
            (1083, 1005, "云浮市", "Yunfu"),
            (1084, 1005, "湛江市", "Zhanjiang"),
            (1085, 1005, "肇庆市", "Zhaoqing"),
            (1086, 1005, "中山市", "Zhongshan"),
            (1087, 1005, "珠海市", "Zhuhai"),
            (1088, 1005, "潮州市", "Chaozhou"),
            (1089, 1005, "东莞市", "Dongguan"),
            (1090, 1005, "佛山市", "Foshan"),
            (1091, 1005, "广州市", "Guangzhou"),
            (1092, 1005, "河源市", "Heyuan"),
            (1093, 1005, "惠州市", "Huizhou"),
            (1094, 1005, "揭阳市", "Jieyang"),
            (1095, 1005, "江门市", "Jiangmen"),
            (1096, 1025, "柳州市", "Liuzhou"),
            (1097, 1025, "崇左市", "Congzuo"),
            (1098, 1025, "钦州市", "Qinzhou"),
            (1099, 1025, "梧州市", "Wuzhou"),
            (1100, 1025, "玉林市", "Yulin"),
            (1101, 1025, "南宁市", "Nanning"),
            (1102, 1025, "来宾市", "Laibin"),
            (1103, 1025, "百色市", "Baise"),
            (1104, 1025, "北海市", "Beihai"),
            (1105, 1025, "防城港市", "Fangchengguang"),
            (1106, 1025, "桂林市", "Guilin"),
            (1107, 1025, "贵港市", "Guigang"),
            (1108, 1025, "河池地区", "Hechi"),
            (1109, 1025, "贺州地区", "Hezhou"),
            (1110, 1021, "六盘水市", "Liupanshui"),
            (1111, 1021, "黔西南布依族苗族自治州", "Qian Southwest"),
            (1112, 1021, "黔南布依族苗族自治州", "Qian South"),
            (1113, 1021, "黔东南苗族侗族自治州", "Qian Southeast"),
            (1114, 1021, "铜仁地区", "Tongren"),
            (1115, 1021, "安顺市", "Anshun"),
            (1116, 1021, "毕节地区", "Bijie"),
            (1117, 1021, "贵阳市", "Guiyang"),
            (1118, 1021, "遵义市", "Zunyi"),
            (1119, 1019, "三亚市", "Sanya"),
            (1120, 1019, "海口市", "Haikou"),
            (1121, 1007, "廊坊市", "Langfang"),
            (1122, 1007, "秦皇岛市", "Qinhuangdao"),
            (1123, 1007, "邢台市", "Xingtai"),
            (1124, 1007, "石家庄市", "Shijiazhuang"),
            (1125, 1007, "唐山市", "Tangshan"),
            (1126, 1007, "张家口市", "Zhangjiakou"),
            (1127, 1007, "保定市", "Baoding"),
            (1128, 1007, "沧州市", "Cangzhou"),
            (1129, 1007, "承德市", "Chengde"),
            (1130, 1007, "衡水市", "Hengshui"),
            (1131, 1007, "邯郸市", "Handan"),
            (1132, 1010, "洛阳市", "Luoyang"),
            (1133, 1010, "新乡市", "Xinxiang"),
            (1134, 1010, "许昌市", "Xuchang"),
            (1135, 1010, "信阳市", "Xinyang"),
            (1136, 1010, "商丘市", "Shangqiu"),
            (1137, 1010, "三门峡市", "Sanmenxia"),
            (1138, 1010, "濮阳市", "Puyang"),
            (1139, 1010, "漯河市", "Luohe"),
            (1140, 1010, "南阳市", "Nanyang"),
            (1141, 1010, "平顶山市", "Pingdingshan"),
            (1142, 1010, "周口市", "Zhoukou"),
            (1143, 1010, "郑州市", "Zhengzhou"),
            (1144, 1010, "安阳市", "Anyang"),
            (1145, 1010, "鹤壁市", "Hebi"),
            (1146, 1010, "焦作市", "Jiaozuo"),
            (1147, 1010, "开封市", "Kaifeng"),
            (1148, 1010, "驻马店市", "Zhumadian"),
            (1149, 1013, "绥化市", "Suihua"),
            (1150, 1013, "双鸭山市", "Shuangyashan"),
            (1151, 1013, "伊春市", "Yichun"),
            (1152, 1013, "齐齐哈尔市", "Qiqihar"),
            (1153, 1013, "牡丹江市", "Mudanjiang"),
            (1154, 1013, "七台河市", "Qitaihe"),
            (1155, 1013, "大庆市", "Daqing"),
            (1156, 1013, "大兴安岭地区", "Daxinganling"),
            (1157, 1013, "哈尔滨市", "Harbing"),
            (1158, 1013, "黑河市", "Heihe"),
            (1159, 1013, "鹤岗市", "Hegang"),
            (1160, 1013, "佳木斯市", "Jiamusi"),
            (1161, 1013, "鸡西市", "Jixi"),
            (1162, 1017, "孝感市", "Xiaogan"),
            (1163, 1017, "随州市", "Suizhou"),
            (1164, 1017, "十堰市", "Shiyan"),
            (1165, 1017, "咸宁市", "Xianning"),
            (1166, 1017, "襄樊市", "Xiangfan"),
            (1167, 1017, "武汉市", "Wuhan"),
            (1168, 1017, "宜昌市", "Yichang"),
            (1169, 1017, "鄂州市", "Ezhou"),
            (1170, 1017, "恩施市", "Enshi"),
            (1171, 1017, "黄石市", "Huangshi"),
            (1172, 1017, "黄冈市", "Huanggang"),
            (1173, 1017, "荆门市", "Jingmen"),
            (1174, 1017, "荆州市", "Jingzhou"),
            (1175, 1018, "娄底市", "Loudi"),
            (1176, 1018, "邵阳市", "Shaoyang"),
            (1177, 1018, "湘潭市", "Xiangtan"),
            (1178, 1018, "湘西土家族苗族自治州", "Xiangxi"),
            (1179, 1018, "岳阳市", "Yueyang"),
            (1180, 1018, "永州市", "Yongzhou"),
            (1181, 1018, "张家界市", "Zhangjiajie"),
            (1182, 1018, "益阳市", "Yiyang"),
            (1183, 1018, "株洲市", "Zhuzhou"),
            (1184, 1018, "常德市", "Changde"),
            (1185, 1018, "长沙市", "Changsha"),
            (1186, 1018, "郴州市", "Chenzhou"),
            (1187, 1018, "衡阳市", "Hengyang"),
            (1188, 1018, "怀化市", "Huaihua"),
            (1189, 1012, "辽源市", "Liaoyuan"),
            (1190, 1012, "松原市", "Songyuan"),
            (1191, 1012, "四平市", "Siping"),
            (1192, 1012, "通化市", "Tonghua"),
            (1193, 1012, "延边朝鲜族自治州", "Yanbian"),
            (1194, 1012, "白山市", "Baishan"),
            (1195, 1012, "白城市", "Baicheng"),
            (1196, 1012, "长春市", "Changchun"),
            (1197, 1012, "吉林市", "Jilin"),
            (1198, 1006, "连云港市", "Lianyungang"),
            (1199, 1006, "徐州市", "Xuzhou"),
            (1200, 1006, "苏州市", "Suzhou"),
            (1201, 1006, "宿迁市", "Suqian"),
            (1202, 1006, "泰州市", "Taizhou"),
            (1203, 1006, "无锡市", "Wuxi"),
            (1204, 1006, "扬州市", "Yangzhou"),
            (1205, 1006, "盐城市", "Yancheng"),
            (1206, 1006, "镇江市", "Zhenjiang"),
            (1207, 1006, "南通市", "Nantong"),
            (1208, 1006, "南京市", "Nanjing"),
            (1209, 1006, "常州市", "Changzhou"),
            (1210, 1006, "淮安市", "Huaian"),
            (1211, 1020, "新余市", "Xinyu"),
            (1212, 1020, "上饶市", "Shangrao"),
            (1213, 1020, "宜春市", "Yichun"),
            (1214, 1020, "鹰潭市", "Yingtan"),
            (1215, 1020, "南昌市", "Nanchang"),
            (1216, 1020, "萍乡市", "Pingxiang"),
            (1217, 1020, "赣州市", "Ganzhou"),
            (1218, 1020, "抚州市", "Fuzhou"),
            (1219, 1020, "吉安市", "Jian"),
            (1220, 1020, "九江市", "Jiujiang"),
            (1221, 1020, "景德镇市", "Jingdezhen"),
            (1222, 1011, "辽阳市", "Liaoyang"),
            (1223, 1011, "沈阳市", "Shenyang"),
            (1224, 1011, "铁岭市", "Tieling"),
            (1225, 1011, "营口市", "Yingkou"),
            (1226, 1011, "盘锦市", "Panjin"),
            (1227, 1011, "鞍山市", "Anshan"),
            (1228, 1011, "本溪市", "Benxi"),
            (1229, 1011, "朝阳市", "Chaoyang"),
            (1230, 1011, "大连市", "Dalian"),
            (1231, 1011, "丹东市", "Dandong"),
            (1232, 1011, "抚顺市", "Fushun"),
            (1233, 1011, "阜新市", "Fuxin"),
            (1234, 1011, "葫芦岛市", "Huludao"),
            (1235, 1011, "锦州市", "Jinzhou"),
            (1236, 1030, "兴安盟", "Xingan"),
            (1237, 1030, "乌兰察布市", "Wulanchabu"),
            (1238, 1030, "乌海市", "Wuhai"),
            (1239, 1030, "锡林郭勒盟", "Xilinguole"),
            (1240, 1030, "通辽市", "Tongliao"),
            (1241, 1030, "巴彦淖尔市", "Bayanzhuoer"),
            (1242, 1030, "包头市", "Baotou"),
            (1243, 1030, "阿拉善盟", "Alashan"),
            (1244, 1030, "赤峰市", "Chifeng"),
            (1245, 1030, "鄂尔多斯市", "Erduosi"),
            (1246, 1030, "呼伦贝尔市", "Hulunbeir"),
            (1247, 1030, "呼和浩特市", "Huhehaote"),
            (1248, 1026, "石嘴山市", "Shizuishan"),
            (1249, 1026, "吴忠市", "Wuzhong"),
            (1250, 1026, "银川市", "Yinchuan"),
            (1251, 1026, "固原市", "Guyuan"),
            (1252, 1027, "西宁市", "Xining"),
            (1253, 1027, "玉树藏族自治州", "Yushu"),
            (1254, 1027, "果洛藏族自治州", "Guoluo"),
            (1255, 1027, "海西蒙古族藏族自治州", "Haixi"),
            (1256, 1027, "海南藏族自治州", "Hainan"),
            (1257, 1027, "海北藏族自治州", "Haibei"),
            (1258, 1027, "海东地区", "Haidong"),
            (1259, 1027, "黄南藏族自治州", "Huangnan"),
            (1260, 1014, "临沂市", "Linyi"),
            (1261, 1014, "聊城市", "Liaocheng"),
            (1262, 1014, "莱芜市", "Laiwu"),
            (1263, 1014, "青岛市", "Qingdao"),
            (1264, 1014, "日照市", "Rizhao"),
            (1265, 1014, "泰安市", "Taian"),
            (1266, 1014, "潍坊市", "Weifang"),
            (1267, 1014, "威海市", "Weihai"),
            (1268, 1014, "烟台市", "Yantai"),
            (1269, 1014, "枣庄市", "Zaozhuang"),
            (1270, 1014, "滨州市", "Binzhou"),
            (1271, 1014, "德州市", "Dezhou"),
            (1272, 1014, "东营市", "Dongying"),
            (1273, 1014, "菏泽市", "Heze"),
            (1274, 1014, "济南市", "Jinan"),
            (1275, 1014, "济宁市", "Jining"),
            (1276, 1014, "淄博市", "Zibo"),
            (1277, 1008, "吕梁市", "Lvliang"),
            (1278, 1008, "临汾市", "Linfen"),
            (1279, 1008, "忻州市", "Xinzhou"),
            (1280, 1008, "朔州市", "Shuozhou"),
            (1281, 1008, "太原市", "Taiyuan"),
            (1282, 1008, "阳泉市", "Yangquan"),
            (1283, 1008, "运城市", "Yuncheng"),
            (1284, 1008, "长治市", "Changzhi"),
            (1285, 1008, "大同市", "Datong"),
            (1286, 1008, "晋中市", "Jinzhong"),
            (1287, 1008, "晋城市", "Jincheng"),
            (1288, 1023, "商洛市", "Shangluo"),
            (1289, 1023, "咸阳市", "Xianyang"),
            (1290, 1023, "铜川市", "Tongchuan"),
            (1291, 1023, "渭南市", "Weinan"),
            (1292, 1023, "西安市", "Xian"),
            (1293, 1023, "延安市", "Yanan"),
            (1294, 1023, "榆林市", "Yulin"),
            (1295, 1023, "安康市", "Ankang"),
            (1296, 1023, "宝鸡市", "Baoji"),
            (1297, 1023, "汉中市", "Hanzhong"),
            (1298, 1009, "绵阳市", "Mianyang"),
            (1299, 1009, "泸州市", "Luzhou"),
            (1300, 1009, "乐山市", "Leshan"),
            (1301, 1009, "凉山彝族自治州", "Liangshan"),
            (1302, 1009, "眉山市", "Meishan"),
            (1303, 1009, "遂宁市", "Suining"),
            (1304, 1009, "雅安市", "Yaan"),
            (1305, 1009, "宜宾市", "Yibin"),
            (1306, 1009, "攀枝花市", "Panzhihua"),
            (1307, 1009, "南充市", "Nanchong"),
            (1308, 1009, "内江市", "Neijiang"),
            (1309, 1009, "巴中市", "Bazhong"),
            (1310, 1009, "阿坝藏族羌族自治州", "Aba"),
            (1311, 1009, "成都市", "Chengdu"),
            (1312, 1009, "达州市", "Dazhou"),
            (1313, 1009, "德阳市", "Deyang"),
            (1314, 1009, "甘孜藏族自治州", "Ganzi"),
            (1315, 1009, "广元市", "Guangyuan"),
            (1316, 1009, "广安市", "Guangan"),
            (1317, 1009, "资阳市", "Ziyang"),
            (1318, 1009, "自贡市", "Zigong"),
            (1319, 1029, "林芝地区", "Linzhi"),
            (1320, 1029, "拉萨市", "Lasa"),
            (1321, 1029, "日喀则地区", "Rikaze"),
            (1322, 1029, "山南地区", "Shannan"),
            (1323, 1029, "那曲地区", "Naqu"),
            (1324, 1029, "阿里地区", "Ali"),
            (1325, 1029, "昌都地区", "Changdu"),
            (1326, 1028, "吐鲁番市", "Tulufan"),
            (1327, 1028, "乌鲁木齐市", "Wulumuqi"),
            (1328, 1028, "伊宁市", "Yining"),
            (1329, 1028, "阿图什市", "Atushi"),
            (1330, 1028, "阿克苏市", "Akesu"),
            (1331, 1028, "昌吉市", "Changji"),
            (1332, 1028, "博乐市", "Bole"),
            (1333, 1028, "哈密市", "Hami"),
            (1334, 1028, "和田市", "Hetian"),
            (1335, 1028, "库尔勒市", "Kuerle"),
            (1336, 1028, "克拉玛依市", "Kelamayi"),
            (1337, 1028, "喀什市", "Kashi"),
            (1338, 1022, "临沧地区", "Lincang"),
            (1339, 1022, "丽江市", "Lijiang"),
            (1340, 1022, "昆明市", "Kunming"),
            (1341, 1022, "潞西市", "Luxi"),
            (1342, 1022, "曲靖市", "Qujing"),
            (1343, 1022, "思茅市", "Simao"),
            (1344, 1022, "文山壮族苗族自治州", "Wenshan"),
            (1345, 1022, "昭通市", "Zhaotong"),
            (1346, 1022, "玉溪市", "Yuxi"),
            (1347, 1022, "怒江傈僳族自治州", "Nujiang"),
            (1348, 1022, "保山市", "Baoshan"),
            (1349, 1022, "楚雄市", "Chuxiong"),
            (1350, 1022, "大理市", "Dali"),
            (1351, 1022, "迪庆藏族自治州", "Diqing"),
            (1352, 1022, "个旧市", "Gejiu"),
            (1353, 1022, "景洪市", "Jinghong"),
            (1354, 1004, "衢州市", "Quzhou"),
            (1355, 1004, "丽水市", "Lishui"),
            (1356, 1004, "绍兴市", "Shaoxing"),
            (1357, 1004, "温州市", "Wenzhou"),
            (1358, 1004, "台州市", "Taizhou"),
            (1359, 1004, "宁波市", "Ningbo"),
            (1360, 1004, "舟山市", "Zhoushan"),
            (1361, 1004, "杭州市", "Hangzhou"),
            (1362, 1004, "湖州市", "Huzhou"),
            (1363, 1004, "嘉兴市", "Jiaxing"),
            (1364, 1004, "金华市", "Jinhua"),
            (1365, 1028, "石河子市", "Shihezi"),
            (1366, 1028, "阿拉尔市", "Alaer"),
            (1367, 1028, "图木舒克市", "Tumushuke"),
            (1368, 1028, "五加渠市", "Wujiaqu"),
            (1369, 1000, "北京市", "Beijing"),
            (1370, 1001, "上海市", "Shanghai"),
            (1371, 1002, "天津市", "Tianjin"),
            (1372, 1003, "重庆市", "Chongqing"),
            (1373, 1031, "香港", "Hongkong"),
            (1374, 1032, "澳门", "Macao"),
            (1375, 1033, "台湾", "Taiwan"),
            (1392, 1019, "白沙黎族自治县", "Baisha"),
            (1393, 1019, "保亭黎族苗族自治县", "Baoting"),
            (1394, 1019, "昌江黎族自治县", "Changjiang"),
            (1395, 1019, "澄迈县", "Chengmai"),
            (1396, 1019, "定安县", "Dingan"),
            (1397, 1019, "东方市", "Dongfang"),
            (1398, 1019, "乐东黎族自治县", "Ledong"),
            (1399, 1019, "临高县", "Lingao"),
            (1400, 1019, "陵水黎族自治县", "Lingshui"),
            (1401, 1019, "琼海市", "Qionghai"),
            (1402, 1019, "琼中黎族苗族自治县", "Qiongzhong"),
            (1403, 1019, "屯昌县", "Tunchang"),
            (1404, 1019, "万宁市", "Wanning"),
            (1405, 1019, "文昌市", "Wenchang"),
            (1406, 1019, "五指山市", "Wuzhishan"),
            (1407, 1019, "儋州市", "Danzhou"),
            (1408, 1010, "济源市", "Jiyuan"),
            (1409, 1017, "潜江市", "Qianjiang"),
            (1410, 1017, "神农架林区", "Shennongjia"),
            (1411, 1017, "天门市", "Tianmen"),
            (1412, 1017, "仙桃市", "Xiantao"),
            (1413, 1025, "凭祥市", "Pinxiang"),
            (3323, 1019, "琼山市", "Qiongshan"),
            (4029, 1024, "玉门市", "Yumen"),
            (4030, 1024, "敦煌市", "Dunhuang"),
            (4274, 4273, "国外", "Abroad "),
        ]
        for c in cs:
            im = self.add_city(int(c[0]), int(c[1]), str(c[2]), str(c[3]))
            if im.error:
                return im
        return im
            
